/*
 * Copyright (c) 2013 - 2016, Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NXP "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NXP OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef ISELED_DRIVER_H_
#define ISELED_DRIVER_H_
#include "timing_pal.h"

/*!
 * @file iseled_driver.h
 *
 * @page misra_violations MISRA-C:2012 violations
 *
 * @section [global]
 * Violates MISRA 2012 Advisory Rule 2.3, A project should not contain unused type declarations.
 * The type is defined here to be used by other layers of the driver.
 *
 * @section [global]
 * Violates MISRA 2012 Advisory Rule 2.5, Global macro not referenced.
 * The macros represent the available pins, used in the configuration process by the application
 *
 */
 
/*!
 * @addtogroup iseled_driver ISELED Driver
 * @ingroup iseled
 * @brief ISELED Driver
 * @{
 */
void timeoutCallback(void *param);

/* Commands supported by the digLED_Send_Cmd_Block function */
#define SET_RGB_BLOCK    1U
#define SET_DIM_BLOCK    2U

/*! @brief Transfer type
 * Implements : digLED_TransferType_Class
 */
typedef enum
{
	ISELED_USING_DMA = 0x00U,
	ISELED_USING_INTERRUPTS
} digLED_TransferType;

/*! @brief Interface selection
 * Implements : digLED_InterfaceType_Class
 */
typedef enum
{
    DIGLED_INTERFACE_FLEXIO = 0x00U,
    DIGLED_INTERFACE_LPSPI    = 0x01U,
    DIGLED_INTERFACE_GPIO   = 0x02U
} digLED_InterfaceType;

/*! @brief Event type
 * Implements : digLED_CommEventType_Class
 */
typedef enum
{
    TRANSMISSION_COMPLETE = 0x01U,
    RECEPTION_COMPLETE = 0x02U,
    RECEPTION_TIMEOUT = 0x03U,
    CRC_ERROR = 0x04U,
    CHAINLENGTH_ERROR = 0x05U
} digLED_CommEventType;

/*! @brief Pin config for a LED strip
 * Implements : digLED_PinConfigType_Class
 */
typedef struct
{
	uint8_t stripNr;
    uint8_t dataPin;
    uint8_t clockPin;
} digLED_PinConfigType;

/*! @brief Timeout configuration for a LED strip
 * Implements : digLED_TimeoutType_Class
 */
typedef struct
{
	const timing_instance_t *timingPalInstance;
	uint32_t          timeoutDuration;
	uint8_t           timerChannel;
} digLED_TimeoutType;

/*! @brief Configuration structure
 * Implements : digLED_ConfigType_Class
 */
typedef struct
{
    uint8_t                        nrOfStrips;
    void                           (*callback)(digLED_CommEventType event, uint8_t strip);
    digLED_InterfaceType           interfaceType;
    digLED_PinConfigType           *pinConfig;
    digLED_TimeoutType             *timeoutConfig;
    digLED_TransferType            transferType;
    uint8_t                        dmaChanTx;
    uint8_t                        dmaChanRx;
} digLED_ConfigType;

/*! @brief Return type
 * Implements : digLED_ReturnType_Class
 */
typedef enum
{
    DIGLED_ERROR    = 0U,  /*!< Function returns with failure */
    DIGLED_OK       = 1U,  /*!< Function returns successful */
	DIGLED_BUSY     = 2U,  /*!< Function is busy*/
	DIGLED_EVALUATION_OVER = 3U, /*!< Functions not available anymore - only on unauthorized MCUs*/
}digLED_ReturnType;

/*! @brief Initialization parameters
 * Implements : digLED_InitType_Class
 */
typedef struct
{
    uint16_t firstLedAdr;                      /*!< Address of the first LED in the chain */
    bool crcEnable;                            /*!< 0:disable, 1:enable CRC generation at frames */
    bool tempCmpEnable;                        /*!< 0:disable, 1:enable temperature compensation for red LED */
	bool voltSwing;                            /*!< 0:250mV, 1:500mV differential voltage swing */
	bool phaseShift;                           /*!< 0:activate the 3 PWM channels phase shift
	                                            *   1:deactivate the 3 PWM channels phase shift */
} digLED_InitType;

/*! @brief Return data
 * Implements : digLED_ReadDataResultType_Class
 */
typedef struct
{
    uint16_t chainLength;                     /*!< 0:4095 Amount of Smart LED drivers of the chain. */
    uint32_t *retData;                        /*!< Pointer to an array with status values of each Smart LED driver of the chain. */
} digLED_ReadDataResultType;

extern const digLED_ReadDataResultType *readResultPtr;

/*! @brief Block command member. These are used to create a list of commands to be sent
 * Implements : digLED_SendCmdBlockType_Class
 */
typedef struct
{
	uint8_t red;
	uint8_t green;
	uint8_t blue;
	uint16_t addr;
	uint8_t cmd;
	uint8_t padding[10U];
} digLED_SendCmdBlockType;

/*!
  * @brief Initiates a block transfer of multiple commands
  *
  * This function sends a block transfer of multiple commands over DMA or Interrupts
  *
  * @param data                    Pointer to the user's buffer of commands
  * @param cmdNr                   Number of commands in the buffer
  * @param stripNr                 Number of the strip on which the commands will be sent
  * @return DIGLED_OK
  *
  */
digLED_ReturnType digLED_Send_Cmd_Block(
	digLED_SendCmdBlockType *data,
	uint32_t cmdNr,
    uint8_t stripNr
);

/*!
  * @brief Initializes the ISELED interface or interfaces
  *
  * This function initializes the ISELED driver
  *
  * @param nrOfInterfaces        Number of hardware interfaces used for ISELED communication (FlexIO / LPSPI)
  * @param configStruct            Pointer to the configuration structure
  * @return STATUS_SUCCESS
  *
  */
digLED_ReturnType digLED_Init_Interface(uint8_t nrOfInterfaces, const digLED_ConfigType *configStruct);

/*!
  * @brief Initializes on LED strip
  *
  * This function initializes one LED strip for communication
  *
  * @param type                    Type of hardware interface to be used (FlexIO / LPSPI)
  * @param configStruct            Pointer to the configuration structure
  * @return Nothing
  * 
  */
digLED_ReturnType digLED_Init_Strip(const digLED_InitType* ChainInitPtr, const digLED_ReadDataResultType* ChainInitResultPtr, uint8_t stripNr);

/*!
  * @brief Set a RGB value to the addressed LED
  *
  * This function sets the color of the addressed LED. Address = 0 is a broadcast address targeting all LEDs in parallel.
  *
  * @param Red                    0-255: Sets the intensity of the red LED
  * @param Green                0-255: Sets the intensity of the green LED
  * @param Blue                    0-255: Sets the intensity of the blue LED
  * @param Address                0-4095: Address of the target LED. 0 addresses all LEDs of the chain.
  * @return Nothing
  * 
  */
digLED_ReturnType digLED_Set_RGB
(
    uint8_t Red,
    uint8_t Green,
    uint8_t Blue,
    uint16_t Address,
    uint8_t stripNr
);

/*!
  * @brief Triggers a measurement specified by TestNr.
  *
  * This function triggers the measurement of an chip internal parameter specified by TestNr. The result of the measurement is temporarily
  * stored and can be accessed by a read command.
  *
  * @param TestNr                1: Measure reference voltage 1 (5V)
  *                             2: Measure reference voltage 2 (1.5V)
  *                             3: Measure Vf red LED
  *                             4: Measure Vf green LED
  *                             5: Measure Vf blue LED
  *                             6: Measure bias voltage
  * @param Address                0-4095: Address of the target LED. 0 addresses all LEDs of the chain.
  * @return Nothing
  * 
  */
digLED_ReturnType digLED_Test (uint8_t TestNr, uint16_t Address, uint8_t stripNr);

 /*!
  * @brief Triggers the software reset
  * 
  */
digLED_ReturnType digLED_Reset (uint8_t stripNr);

 /*!
   * @brief Configure the addressed Smart LED driver.
   *
   * Sets the 12 bit configuration value of the addressed Smart LED driver.
   *
   * @param Param                0-4095: 12 bit configuration value
   * @param Address                0-4095: Address of the target LED. 0 addresses all LEDs of the chain.
   * @return Nothing
   * 
   */
digLED_ReturnType digLED_Set_Config (uint16_t Param, uint16_t Address, uint8_t stripNr);

 /*!
  * @brief Set max PWM value for red LED
  *
  * Set max PWM value for red LED of the addressed Smart LED driver.
  *
  * @param Param                0-4095: 12 bit PWM value
  * @param Address                0-4095: Address of the target LED. 0 addresses all LEDs of the chain.
  * @return Nothing
  * 
  */
digLED_ReturnType digLED_Set_PWM_Red(uint16_t Param, uint16_t Address, uint8_t stripNr);

/*!
  * @brief Set max PWM value for green LED
  *
  * Set max PWM value for green LED of the addressed Smart LED driver.
  *
  * @param Param                0-4095: 12 bit PWM value
  * @param Address                0-4095: Address of the target LED. 0 addresses all LEDs of the chain.
  * @return Nothing
  * 
  */
digLED_ReturnType digLED_Set_PWM_Green(uint16_t Param, uint16_t Address, uint8_t stripNr);

/*!
  * @brief Set max PWM value for blue LED
  *
  * Set max PWM value for blue LED of the addressed Smart LED driver.
  *
  * @param Param                0-4095: 12 bit PWM value
  * @param Address                0-4095: Address of the target LED. 0 addresses all LEDs of the chain.
  * @return Nothing
  * 
  */
digLED_ReturnType digLED_Set_PWM_Blue(uint16_t Param, uint16_t Address, uint8_t stripNr);

 /*!
  * @brief Set the peak current value for green LED
  *
  * Set peak current value for green LED of the addressed Smart LED driver and drives the peak current for 25ms
  *
  * @param Param                0-15: 4 bit current value
  * @param Address                0-4095: Address of the target LED. 0 addresses all LEDs of the chain.
  * @return Nothing
  * 
  */
digLED_ReturnType digLED_Set_Cur_Green(uint16_t Param, uint16_t Address, uint8_t stripNr);

 /*!
  * @brief Set the peak current value for blue LED
  *
  * Set peak current value for blue LED of the addressed Smart LED driver and drives the peak current for 25ms
  *
  * @param Param                0-15: 4 bit current value#ifdef DIGLED_TEST
  * @param Address                0-4095: Address of the target LED. 0 addresses all LEDs of the chain.
  * @return Nothing
  * 
  */
digLED_ReturnType digLED_Set_Cur_Blue(uint16_t Param, uint16_t Address, uint8_t stripNr);

 /*!
  * @brief Set temperature offset value
  *
  * Set temperature offset value of the addressed Smart LED driver.
  *
  * @param Param                0-511: 9 bit offset value
  * @param Address                0-4095: Address of the target LED. 0 addresses all LEDs of the chain.
  * @return Nothing
  * 
  */
digLED_ReturnType digLED_Set_Temp_Offset(uint16_t Param, uint16_t Address, uint8_t stripNr);

 /*!
  * @brief Trigger ADC self calibration
  *
  * Trigger ADC self calibration of the addressed Smart LED driver
  *
  * @param Mode                    0: Trigger ADC offset self calibration only
  *                             1: Trigger ADC offset + Vref self calibration
  * @param Address                0-4095: Address of the target LED. 0 addresses all LEDs of the chain.
  * @return Nothing
  * 
  */
digLED_ReturnType digLED_Set_Trg_Adc_Cal(uint8_t Mode, uint16_t Address, uint8_t stripNr);

 /*!
  * @brief Set BIAS value
  *
  * Set BIAS value of the addressed Smart LED driver.
  *
  * @param Param                  0-15: 4 bit BIAS value
  * @param Address                0-4095: Address of the target LED. 0 addresses all LEDs of the chain.
  * @return Nothing
  * 
  */
digLED_ReturnType digLED_Set_Bias(uint8_t Param, uint16_t Address, uint8_t stripNr);

 /*!
  * @brief Set temperature compensation look-up table initialization base value
  *
  * Set temperature compensation look-up table initialization base value of the addressed Smart LED driver.
  *
  * @param Param                0-511: 9 bit TC base value
  * @param Address                0-4095: Address of the target LED. 0 addresses all LEDs of the chain.
  * @return Nothing
  * 
  */
digLED_ReturnType digLED_Set_TC_Base (uint16_t Param, uint16_t Address, uint8_t stripNr);

 /*!
 * @brief Set temperature compensation look-up table initialization offset value
 *
 * Set temperature compensation look-up table initialization offset value of the addressed Smart LED driver.
 *
 * @param Param                0-511: 9 bit TC offset value
 * @param Address                0-4095: Address of the target LED. 0 addresses all LEDs of the chain.
 * @return Nothing
 * 
 */
digLED_ReturnType digLED_Set_TC_Offset (uint16_t Param, uint16_t Address, uint8_t stripNr);

 /*!
  * @brief Set A/D converter R2R D/A value
  *
  * Set DAC value (9 bit) for the addressed Smart LED driver
  *
  * @param Param                  0-511: 9 bit DAC Value
  * @param Address                0-4095: Address of the target LED. 0 addresses all LEDs of the chain.
  * @return Nothing
  * 
  */
digLED_ReturnType digLED_Set_Adc_Dac (uint16_t Param, uint16_t Address, uint8_t stripNr);

 /*!
  * @brief Set temperature compensation look-up table entry for red LED
  *
  * Set temperature compensation look-up table entry for red LED of the addressed Smart LED driver.
  * LUT is automatically initialized with linearly increasing values (see: _TC_Base and _TC_Offset).
  * These setting can be manually overwritten with any values.
  *
  * @param LUT_Adr                0-10: 4 bit look-up table entry (address)
  * @param LUT_Value            0-511: 9 bit look-up table value
  * @param Address                0-4095: Address of the target LED. 0 addresses all LEDs of the chain.
  * @return Nothing
  * 
  */
digLED_ReturnType digLED_Set_TC_Lookup (uint16_t LUT_Adr,
                            uint16_t LUT_Value,
                            uint16_t Address,
                            uint8_t stripNr);

 /*!
  * @brief Get temperature of all LEDs
  *
  * Reads the temperature of all LEDs in chain and stores the data in a structure of type digLED_ReadDataResultType.
  *
  * @param ChainTempPtr                Pointer to structure containing the temperature values of all LEDs in the chain.
  * @return DIGLED_OK
  * 
  */
 digLED_ReturnType digLED_Read_Temp (const digLED_ReadDataResultType* ChainTempPtr, uint8_t stripNr);

 /*!
  * @brief Get status of all LEDs
  *
  * Reads the status of all LEDs in chain and stores the data in a structure of type digLED_ReadDataResultType.
  *
  * @param ChainStatusPtr            Pointer to structure containing the status values of all LEDs in the chain.
  * @return DIGLED_OK
  * 
  */
 digLED_ReturnType digLED_Read_Status (const digLED_ReadDataResultType* ChainStatusPtr, uint8_t stripNr);

 /*!
  * @brief Get diagnostic value of all LEDs
  *
  * Reads the diagnostic values of all LEDs in chain and stores the data in a structure of type digLED_ReadDataResultType.
  * The diagnostic value is the value which is obtained by a call of the digLED_Test function.
  *
  * @param ChainDiagPtr                Pointer to structure containing the diagnostic values of all LEDs in the chain.
  * @return DIGLED_OK
  * 
  */
 digLED_ReturnType digLED_Read_Diagnostic (const digLED_ReadDataResultType* ChainDiagPtr, uint8_t stripNr);

/*!
 * @brief Get diagnostic value of all LEDs
 *
 * Reads the parameter specified by ParamNumber of all LEDs in chain and stores the data in a structure of type digLED_ReadDataResultType.
 *
 * @param ParamNumber              1: configuration
 *                                 2: max PWM value for red LED
 *                                 3: max PWM value for green LED
 *                                 4: max PWM value for blue LED
 *                                 5: peak current for green LED
 *                                 6: peak current for blue LED
 *                                 7: temperature sensor offset value
 *                                 8: ADC offset and Vref cal. values
 *                                 9: bias reference
 *                                 10: temp. comp. LUT init. base value
 *                                 11: temp. comp. LUT init. offset value
 *                                 15: last fuse
 * @param ChainStatusPtr            Pointer to structure containing the status values of all LEDs in the chain.
 * @return DIGLED_OK
 * 
 */
digLED_ReturnType digLED_Read_Param (uint16_t ParamNumber, const digLED_ReadDataResultType* ChainStatusPtr, uint8_t stripNr);

/*!
 * @brief Dim, shift the final PWM values right by the given amount of bits
 *
 *@param (in)
 *                Red: amount of bits to shift (0..3) the red LED PWM value
 *                Green: amount of bits to shift (0..3) the green LED PWM value
 *                Blue: amount of bits to shift (0..3) the blue LED PWM value
 *                 Address
 *                      0-4095: Address of the target LED.
 *                        0 addresses all LEDs of the chain.
 *
 * @return DIGLED_OK
 * 
 */
digLED_ReturnType digLED_Set_Dim(uint8_t Red,
                uint8_t Green,
                uint8_t Blue,
                uint16_t Address,
                uint8_t stripNr);


/*!
 * @brief Define multicast groups
 *                u: lower/upper 8 groups, 0: define lower 8 groups, 1: define upper 8 groups
 *                g: group membership, 0: do not join group, 1: join group
 *
 * @param (in)
 *        Param 0-31: 5 bit BIAS value
 *        Address 0-4095: Address of the target LED.
 *                0 addresses all LEDs of the chain.
 * @return DIGLED_OK
 * 
 */
digLED_ReturnType digLED_Define_Mcast(uint16_t Param, uint16_t Address, uint8_t stripNr);

/*!
 * @brief Read current PWM value for the red LED
 *
 *        @param ChainStatusPtr            Pointer to structure containing the values of all LEDs in the chain.
 *
 * @return DIGLED_OK
 * 
 */
digLED_ReturnType digLED_Read_PWM_Red (const digLED_ReadDataResultType* ChainTempPtr, uint8_t stripNr);

/*!
 * @brief Read current PWM value for the green LED
 *
 *        @param ChainStatusPtr            Pointer to structure containing the values of all LEDs in the chain.
 *
 * @return DIGLED_OK
 * 
 */
digLED_ReturnType digLED_Read_PWM_Green (const digLED_ReadDataResultType* ChainTempPtr, uint8_t stripNr);

/*!
 * @brief Read current PWM value for the blue LED
 *
 *        @param ChainStatusPtr            Pointer to structure containing the values of all LEDs in the chain.
 *
 * @return DIGLED_OK
 * 
 */
digLED_ReturnType digLED_Read_PWM_Blue (const digLED_ReadDataResultType* ChainTempPtr, uint8_t stripNr);

/*!
 * @brief Ping the last node in the chain
 *
 * @return DIGLED_OK
 * 
 */
digLED_ReturnType digLED_Ping(const digLED_ReadDataResultType* ChainTempPtr, uint8_t stripNr);

/*! @}*/
#endif /* ISELED_DRIVER_H_ */
